@@ -30,6 +30,8 @@
 //! events. It supports serialization and deserialization but defers logging IO
 //! choices to applications.
 //!
+//! Serialization operates in either a [buffered mode] or a [streaming mode].
+//!
 //! The crate uses Serde for conversion between Rust and JSON.
 //!
 //! [main logging schema]: https://datatracker.ietf.org/doc/html/draft-ietf-quic-qlog-main-schema
@@ -37,6 +39,8 @@
 //! https://datatracker.ietf.org/doc/html/draft-ietf-quic-qlog-quic-events.html
 //! [HTTP/3 and QPACK event definitions]:
 //! https://datatracker.ietf.org/doc/html/draft-ietf-quic-qlog-h3-events.html
+//! [buffered mode]: #buffered-traces-with-standard-json
+//! [streaming mode]: #streaming-traces-with-json-seq
 //!
 //! Overview
 //! ---------------
@@ -107,7 +111,7 @@
 //!
 //! let pkt_hdr = qlog::events::quic::PacketHeader::new(
 //!     qlog::events::quic::PacketType::Initial,
-//!     0,                // packet_number
+//!     Some(0),          // packet_number
 //!     None,             // flags
 //!     None,             // token
 //!     None,             // length
@@ -130,13 +134,15 @@
 //! let event_data =
 //!     qlog::events::EventData::PacketSent(qlog::events::quic::PacketSent {
 //!         header: pkt_hdr,
-//!         frames: Some(frames),
+//!         frames: Some(frames.into()),
 //!         is_coalesced: None,
 //!         retry_token: None,
 //!         stateless_reset_token: None,
 //!         supported_versions: None,
 //!         raw: Some(raw),
 //!         datagram_id: None,
+//!         send_at_time: None,
+//!         trigger: None,
 //!     });
 //!
 //! trace.push_event(qlog::events::Event::with_time(0.0, event_data));
@@ -211,7 +217,7 @@
 //! }
 //! ```
 //!
-//! ## Streaming Traces JSON Text Sequences (JSON-SEQ)
+//! ## Streaming Traces with JSON-SEQ
 //!
 //! To help support streaming serialization of qlogs,
 //! draft-ietf-quic-qlog-main-schema-01 introduced support for RFC 7464 JSON
@@ -281,64 +287,12 @@
 //! streamer.start_log().ok();
 //! ```
 //!
-//! ### Adding simple events
+//! ### Adding events
 //!
-//! Once logging has started you can stream events. Simple events
-//! can be written in one step using [`add_event()`]:
-//!
-//! ```
-//! # let mut trace = qlog::TraceSeq::new(
-//! #    qlog::VantagePoint {
-//! #        name: Some("Example client".to_string()),
-//! #        ty: qlog::VantagePointType::Client,
-//! #        flow: None,
-//! #    },
-//! #    Some("Example qlog trace".to_string()),
-//! #    Some("Example qlog trace description".to_string()),
-//! #    Some(qlog::Configuration {
-//! #        time_offset: Some(0.0),
-//! #        original_uris: None,
-//! #    }),
-//! #    None,
-//! # );
-//! # let mut file = std::fs::File::create("foo.qlog").unwrap();
-//! # let mut streamer = qlog::streamer::QlogStreamer::new(
-//! #     qlog::QLOG_VERSION.to_string(),
-//! #     Some("Example qlog".to_string()),
-//! #     Some("Example qlog description".to_string()),
-//! #     None,
-//! #     std::time::Instant::now(),
-//! #     trace,
-//! #     qlog::events::EventImportance::Base,
-//! #     Box::new(file),
-//! # );
-//! let event_data = qlog::events::EventData::MetricsUpdated(
-//!     qlog::events::quic::MetricsUpdated {
-//!         min_rtt: Some(1.0),
-//!         smoothed_rtt: Some(1.0),
-//!         latest_rtt: Some(1.0),
-//!         rtt_variance: Some(1.0),
-//!         pto_count: Some(1),
-//!         congestion_window: Some(1234),
-//!         bytes_in_flight: Some(5678),
-//!         ssthresh: None,
-//!         packets_in_flight: None,
-//!         pacing_rate: None,
-//!     },
-//! );
-//!
-//! let event = qlog::events::Event::with_time(0.0, event_data);
-//! streamer.add_event(event).ok();
-//! ```
-//!
-//! ### Adding events with frames
-//! Some events contain optional arrays of QUIC frames. If the
-//! event has `Some(Vec<QuicFrame>)`, even if it is empty, the
-//! streamer enters a frame serializing mode that must be
-//! finalized before other events can be logged.
-//!
-//! In this example, a `PacketSent` event is created with an
-//! empty frame array and frames are written out later:
+//! Once logging has started you can stream events. Events
+//! are written in one step using one of [`add_event()`],
+//! [`add_event_with_instant()`], [`add_event_now()`],
+//! [`add_event_data_with_instant()`], or [`add_event_data_now()`] :
 //!
 //! ```
 //! # let mut trace = qlog::TraceSeq::new(
@@ -372,22 +326,33 @@
 //!
 //! let pkt_hdr = qlog::events::quic::PacketHeader::with_type(
 //!     qlog::events::quic::PacketType::OneRtt,
-//!     0,
+//!     Some(0),
 //!     Some(0x00000001),
 //!     Some(&scid),
 //!     Some(&dcid),
 //! );
 //!
+//! let ping = qlog::events::quic::QuicFrame::Ping {
+//!     length: None,
+//!     payload_length: None,
+//! };
+//! let padding = qlog::events::quic::QuicFrame::Padding {
+//!     length: None,
+//!     payload_length: 1234,
+//! };
+//!
 //! let event_data =
 //!     qlog::events::EventData::PacketSent(qlog::events::quic::PacketSent {
 //!         header: pkt_hdr,
-//!         frames: Some(vec![]),
+//!         frames: Some(vec![ping, padding].into()),
 //!         is_coalesced: None,
 //!         retry_token: None,
 //!         stateless_reset_token: None,
 //!         supported_versions: None,
 //!         raw: None,
 //!         datagram_id: None,
+//!         send_at_time: None,
+//!         trigger: None,
 //!     });
 //!
 //! let event = qlog::events::Event::with_time(0.0, event_data);
@@ -395,48 +360,7 @@
 //! streamer.add_event(event).ok();
 //! ```
 //!
-//! In this example, the frames contained in the QUIC packet
-//! are PING and PADDING. Each frame is written using the
-//! [`add_frame()`] method. Frame writing is concluded with
-//! [`finish_frames()`].
-//!
-//! ```
-//! # let mut trace = qlog::TraceSeq::new(
-//! #    qlog::VantagePoint {
-//! #        name: Some("Example client".to_string()),
-//! #        ty: qlog::VantagePointType::Client,
-//! #        flow: None,
-//! #    },
-//! #    Some("Example qlog trace".to_string()),
-//! #    Some("Example qlog trace description".to_string()),
-//! #    Some(qlog::Configuration {
-//! #        time_offset: Some(0.0),
-//! #        original_uris: None,
-//! #    }),
-//! #    None,
-//! # );
-//! # let mut file = std::fs::File::create("foo.qlog").unwrap();
-//! # let mut streamer = qlog::streamer::QlogStreamer::new(
-//! #     qlog::QLOG_VERSION.to_string(),
-//! #     Some("Example qlog".to_string()),
-//! #     Some("Example qlog description".to_string()),
-//! #     None,
-//! #     std::time::Instant::now(),
-//! #     trace,
-//! #     qlog::events::EventImportance::Base,
-//! #     Box::new(file),
-//! # );
-//!
-//! let ping = qlog::events::quic::QuicFrame::Ping;
-//! let padding = qlog::events::quic::QuicFrame::Padding;
-//!
-//! streamer.add_frame(ping, false).ok();
-//! streamer.add_frame(padding, false).ok();
-//!
-//! streamer.finish_frames().ok();
-//! ```
-//!
-//! Once all events have have been written, the log
+//! Once all events have been written, the log
 //! can be finalized with [`finish_log()`]:
 //!
 //! ```
@@ -481,11 +405,13 @@
 //! [`push_event()`]: struct.Trace.html#method.push_event
 //! [`QlogStreamer`]: struct.QlogStreamer.html
 //! [`Write`]: https://doc.rust-lang.org/std/io/trait.Write.html
-//! [`start_log()`]: struct.QlogStreamer.html#method.start_log
-//! [`add_event()`]: struct.QlogStreamer.html#method.add_event
-//! [`add_frame()`]: struct.QlogStreamer.html#method.add_frame
-//! [`finish_frames()`]: struct.QlogStreamer.html#method.finish_frames
-//! [`finish_log()`]: struct.QlogStreamer.html#method.finish_log
+//! [`start_log()`]: streamer/struct.QlogStreamer.html#method.start_log
+//! [`add_event()`]: streamer/struct.QlogStreamer.html#method.add_event
+//! [`add_event_with_instant()`]: streamer/struct.QlogStreamer.html#method.add_event_with_instant
+//! [`add_event_now()`]: streamer/struct.QlogStreamer.html#method.add_event_now
+//! [`add_event_data_with_instant()`]: streamer/struct.QlogStreamer.html#method.add_event_data_with_instant
+//! [`add_event_data_now()`]: streamer/struct.QlogStreamer.html#method.add_event_data_now
+//! [`finish_log()`]: streamer/struct.QlogStreamer.html#method.finish_log
 
 use crate::events::quic::PacketHeader;
 use crate::events::Event;
@@ -512,7 +438,7 @@ pub enum Error {
 
 impl std::fmt::Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
-        write!(f, "{:?}", self)
+        write!(f, "{self:?}")
     }
 }
 
@@ -531,6 +457,7 @@ impl std::convert::From<std::io::Error> for Error {
 pub const QLOG_VERSION: &str = "0.3";
 
 pub type Bytes = String;
+pub type StatelessResetToken = Bytes;
 
 /// A specialized [`Result`] type for quiche qlog operations.
 ///
@@ -586,9 +513,9 @@ pub struct Trace {
     pub events: Vec<Event>,
 }
 
-/// Helper functions for using a qlog `Trace`.
+/// Helper functions for using a qlog [Trace].
 impl Trace {
-    /// Creates a new qlog trace
+    /// Creates a new qlog [Trace]
     pub fn new(
         vantage_point: VantagePoint, title: Option<String>,
         description: Option<String>, configuration: Option<Configuration>,
@@ -604,6 +531,7 @@ impl Trace {
         }
     }
 
+    /// Append an [Event] to a [Trace]
     pub fn push_event(&mut self, event: Event) {
         self.events.push(event);
     }
@@ -621,9 +549,9 @@ pub struct TraceSeq {
     pub common_fields: Option<CommonFields>,
 }
 
-/// Helper functions for using a qlog `TraceSeq`.
+/// Helper functions for using a qlog [TraceSeq].
 impl TraceSeq {
-    /// Creates a new qlog trace
+    /// Creates a new qlog [TraceSeq]
     pub fn new(
         vantage_point: VantagePoint, title: Option<String>,
         description: Option<String>, configuration: Option<Configuration>,
@@ -640,7 +568,7 @@ impl TraceSeq {
 }
 
 #[serde_with::skip_serializing_none]
-#[derive(Serialize, Deserialize, Clone, PartialEq, Debug)]
+#[derive(Serialize, Deserialize, Clone, PartialEq, Eq, Debug)]
 pub struct VantagePoint {
     pub name: Option<String>,
 
@@ -650,7 +578,7 @@ pub struct VantagePoint {
     pub flow: Option<VantagePointType>,
 }
 
-#[derive(Serialize, Deserialize, Clone, PartialEq, Debug)]
+#[derive(Serialize, Deserialize, Clone, PartialEq, Eq, Debug)]
 #[serde(rename_all = "snake_case")]
 pub enum VantagePointType {
     Client,
@@ -683,29 +611,27 @@ pub struct CommonFields {
     pub group_id: Option<String>,
     pub protocol_type: Option<Vec<String>>,
 
-    pub reference_time: Option<f32>,
+    pub reference_time: Option<f64>,
     pub time_format: Option<String>,
     // TODO: additionalUserSpecifiedProperty
 }
 
-#[derive(Serialize, Deserialize, Clone, PartialEq, Debug)]
+#[derive(Serialize, Deserialize, Clone, PartialEq, Eq, Debug)]
 #[serde(rename_all = "snake_case")]
 pub enum TokenType {
     Retry,
     Resumption,
-    StatelessReset,
 }
 
 #[serde_with::skip_serializing_none]
-#[derive(Clone, Serialize, Deserialize, PartialEq, Debug)]
+#[derive(Clone, Serialize, Deserialize, PartialEq, Eq, Debug)]
 pub struct Token {
     #[serde(rename(serialize = "type"))]
     pub ty: Option<TokenType>,
 
-    pub length: Option<u32>,
-    pub data: Option<Bytes>,
-
     pub details: Option<String>,
+
+    pub raw: Option<events::RawInfo>,
 }
 
 pub struct HexSlice<'a>(&'a [u8]);
@@ -726,10 +652,10 @@ impl<'a> HexSlice<'a> {
     }
 }
 
-impl<'a> std::fmt::Display for HexSlice<'a> {
+impl std::fmt::Display for HexSlice<'_> {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         for byte in self.0 {
-            write!(f, "{:02x}", byte)?;
+            write!(f, "{byte:02x}")?;
         }
         Ok(())
     }
@@ -749,7 +675,7 @@ pub mod testing {
 
         PacketHeader::new(
             packet_type,
-            0,
+            Some(0),
             None,
             None,
             None,
@@ -828,7 +754,7 @@ mod tests {
 
         let pkt_hdr = make_pkt_hdr(PacketType::Initial);
         let ev_data = EventData::PacketSent(PacketSent {
-            header: pkt_hdr.clone(),
+            header: pkt_hdr,
             frames: None,
             is_coalesced: None,
             retry_token: None,
@@ -840,6 +766,8 @@ mod tests {
                 data: None,
             }),
             datagram_id: None,
+            send_at_time: None,
+            trigger: None,
         });
 
         let ev = Event::with_time(0.0, ev_data);
@@ -868,7 +796,8 @@ mod tests {
     },
     "frames": [
       {
-        "frame_type": "padding"
+        "frame_type": "padding",
+        "payload_length": 1234
       },
       {
         "frame_type": "ping"
@@ -886,20 +815,27 @@ mod tests {
 
         let pkt_hdr = make_pkt_hdr(PacketType::Initial);
 
-        let mut frames = Vec::new();
-        frames.push(QuicFrame::Padding);
-        frames.push(QuicFrame::Ping);
-        frames.push(QuicFrame::Stream {
-            stream_id: 0,
-            offset: 0,
-            length: 100,
-            fin: Some(true),
-            raw: None,
-        });
+        let frames = vec![
+            QuicFrame::Padding {
+                payload_length: 1234,
+                length: None,
+            },
+            QuicFrame::Ping {
+                payload_length: None,
+                length: None,
+            },
+            QuicFrame::Stream {
+                stream_id: 0,
+                offset: 0,
+                length: 100,
+                fin: Some(true),
+                raw: None,
+            },
+        ];
 
         let ev_data = EventData::PacketSent(PacketSent {
-            header: pkt_hdr.clone(),
-            frames: Some(frames),
+            header: pkt_hdr,
+            frames: Some(frames.into()),
             is_coalesced: None,
             retry_token: None,
             stateless_reset_token: None,
@@ -910,6 +846,8 @@ mod tests {
                 data: None,
             }),
             datagram_id: None,
+            send_at_time: None,
+            trigger: None,
         });
 
         let ev = Event::with_time(0.0, ev_data);
@@ -1017,7 +955,7 @@ mod tests {
         }];
         let event_data = EventData::PacketSent(PacketSent {
             header: pkt_hdr,
-            frames: Some(frames),
+            frames: Some(frames.into()),
             is_coalesced: None,
             retry_token: None,
             stateless_reset_token: None,
@@ -1028,6 +966,8 @@ mod tests {
                 data: None,
             }),
             datagram_id: None,
+            send_at_time: None,
+            trigger: None,
         });
 
         let ev = Event::with_time(0.0, event_data);
@@ -1043,4 +983,5 @@ mod tests {
 }
 
 pub mod events;
+pub mod reader;
 pub mod streamer;
